package info.textgrid.lab.collatex.ui.parts;

import info.textgrid.lab.core.model.TextGridObject;
import info.textgrid.lab.core.swtutils.AdapterUtils;
import info.textgrid.lab.ui.core.utils.TextGridObjectTableViewer;
import info.textgrid.lab.ui.core.utils.TextGridObjectTableViewer.Column;

import java.util.EnumSet;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;

/**
 * A dialog to choose one or more TextGrid object(s)
 * 
 * @TODO should be moved to the generic package
 */
public class SelectObjectDialog extends TitleAreaDialog {
	private Text text;
	private Table table;

	private final String title = Messages.SelectObjectDialog_title;
	private String message = Messages.SelectObjectDialog_message;
	private boolean multi;
	private TextGridObject[] selection;
	private String query;
	private String dialogTitle;


	public boolean isMulti() {
		return multi;
	}

	public void setMulti(final boolean multi) {
		this.multi = multi;
	}

	public TextGridObject[] getSelection() {
		return selection;
	}

	public TextGridObject getFirstElement() {
		return selection.length > 0 ? selection[0] : null;
	}

	public void setSelection(final TextGridObject... selection) {
		this.selection = selection;
	}

	public String getQuery() {
		return query;
	}

	public void setQuery(final String query) {
		this.query = query;
	}

	/**
	 * Create the dialog.
	 * @param parentShell
	 */
	public SelectObjectDialog(final Shell parentShell) {
		super(parentShell);
		setHelpAvailable(false);
	}

	/**
	 * Create contents of the dialog.
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(final Composite parent) {
		setTitle(title);
		setMessage(message);
		getShell().setText(dialogTitle);
		final Composite area = (Composite) super.createDialogArea(parent);
		final Composite container = new Composite(area, SWT.NONE);
		container.setLayout(new GridLayout(1, false));
		container.setLayoutData(new GridData(GridData.FILL_BOTH));

		// text = new Text(container, SWT.BORDER | SWT.H_SCROLL | SWT.SEARCH
		// | SWT.CANCEL);
		// text.setMessage("Filter the results.");
		// text.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1,
		// 1));
		//

		final TextGridObjectTableViewer viewer = new TextGridObjectTableViewer(
				container, (multi ? SWT.MULTI : SWT.SINGLE)
				| SWT.FULL_SELECTION);
		table = viewer.getTable();
		table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		viewer.setVisibleColumns(EnumSet.of(Column.TITLE, Column.DATE,
				Column.AUTHORS));
		viewer.setDefaultSelection(selection);

		if (query != null)
			viewer.queryMetadata(query);

		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				boolean enable = false;

				if (!event.getSelection().isEmpty()) {
					final TextGridObject[] objects = AdapterUtils.getAdapters(
							((IStructuredSelection) event.getSelection()).toArray(),
							TextGridObject.class, false);
					if (objects.length > 0) {
						enable = true;
						selection = objects;
					}
				}
				getButton(IDialogConstants.OK_ID).setEnabled(enable);
			}
		});

		return area;
	}

	public static TextGridObject[] selectObjects(final Shell parentShell,
			final String query,
			final String dialogTitle, final String title, final String message,
			final TextGridObject... initialSelection) {

		final SelectObjectDialog dialog = initDialog(parentShell, query,
				dialogTitle, title, message, initialSelection);
		dialog.setMulti(true);
		dialog.setBlockOnOpen(true);
		final int result = dialog.open();
		if (result == OK)
			return dialog.getSelection();
		else
			return null;
	}

	public static TextGridObject selectObject(final Shell parentShell,
			final String query, final String dialogTitle, final String title,
			final String message, final TextGridObject initialSelection) {

		final SelectObjectDialog dialog = initDialog(parentShell, query,
				dialogTitle, title, message, initialSelection);
		dialog.setMulti(false);
		dialog.setBlockOnOpen(true);
		final int result = dialog.open();
		if (result == OK)
			return dialog.getFirstElement();
		else
			return null;
	}

	private static SelectObjectDialog initDialog(final Shell parentShell,
			final String query, final String dialogTitle, final String title,
			final String message, final TextGridObject... initialSelection) {
		final SelectObjectDialog dialog = new SelectObjectDialog(parentShell);
		if (dialogTitle != null)
			dialog.setDialogTitle(dialogTitle);
		if (title != null)
			dialog.setTitle(title);
		if (message != null)
			dialog.setMessage(message);
		dialog.setQuery(query);
		dialog.setSelection(initialSelection);
		return dialog;
	}

	@Override
	public void setMessage(final String newMessage) {
		this.message = newMessage;
		if (dialogArea != null)
			super.setMessage(newMessage);
	}

	private void setDialogTitle(final String dialogTitle) {
		this.dialogTitle = dialogTitle;
	}

	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(450, 300);
	}
}
